/*
	File:		StdProps.h

	Contains:	Standard Properties

	Written by:	OpenDoc Engineering

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<36>	  2/2/94	RR		Added IsRoot property for frame
		<35>	  2/1/94	CC		Property name changed to
									kXMPPropShouldShowLinks.
		<34>	  2/1/94	SS		Added content property for printing
		<33>	  2/1/94	CC		Added kXMPPropOriginalID,
									kXMPPropOriginalDraft, kXMPPropCloneKind,
									and kXMPPropShowLinkBorders.
		<32>	 1/27/94	VL		Added kXMPPropStandInFrame.
		<31>	 1/26/94	RR		Subrame -> SubFrame
		<30>	 1/21/94	CC		Removed kXMPPropLinkState and
									kXMPPropTimeStamp.  Added kXMPPropLink,
									kXMPPropLinkSource, and kXMPPropChangeID.
		<29>	 1/20/94	SS		Added PartsBin prop
		<28>	 1/14/94	T		removed some Draft Properties, renamed
									others
		<27>	 1/14/94	CC		Added kXMPPropLinkContentSU,
									kXMPPropTimeStamp, kXMPPropAutoExport
		<26>	 1/13/94	JBS		added Frame: kXMPPropIsSubframe
		<25>	12/21/93	JBS		added kXMPPropPresentation
		<24>	12/16/93	CC		Added document properties for linking.
		<23>	12/12/93	T		added kXMPPropPreferredEditor
		<22>	12/11/93	T		added kXMPPropIsStationery
		<21>	 12/8/93	T		added kXMPPropIconFamily for parts and
									kXMPPropName and kXMPPropComments for
									persistent objects
		<20>	11/19/93	PH		Move kXMPPropCurrentSURef here
		<19>	11/18/93	T		added props for Creation, Mod Date and Mod
									User
		<18>	11/17/93	SS		Added Props for Parts
		<17>	11/16/93	EL		Change OpenDocProp to OpenDoc:Property.
		<16>	11/12/93	JBS		add Facet, remove Layout
		<15>	11/10/93	CC		Added kXMPPropLinkSpec and
									kXMPPropSourcePart
		<14>	 11/9/93	PH		Add extra colon
		<13>	10/27/93	eeh		added kXMPPropWindowHasMailer
		<12>	10/21/93	RR		Added window properties isResizable,
									isFloating and isRootWindow
		<11>	10/12/93	eeh		added kXMPPropWindowIsMailerAware
		<10>	 10/4/93	RR		Added window properties
		 <9>	 9/24/93	PH		Added kXMPPropContents
		 <8>	 9/23/93	VL		Added StorageUnitType.
		 <7>	  9/3/93	JBS		added Layout props, fixed some window props
		 <6>	 8/13/93	PH		Name all properties with OpenDocProp to
									avoid conflicts with values
		 <5>	 8/12/93	T		unify property constants
		 <4>	 8/11/93	T		add properties for persistent objects
		 <3>	 8/10/93	T		add properties for WindowState
		 <2>	  8/9/93	VL		Added new storage types.
		 <1>	  7/7/93	VL		first checked in

	To Do:
*/

#ifndef _STDPROPS_
#define _STDPROPS_

#ifndef _STORAGEU_
#include "StorageU.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	This file is used to store all the standard properties.
*/

//==============================================================================
// Constants
//==============================================================================

// Storage
const	XMPPropertyName kXMPPropVersionList 		= "OpenDoc:Property:DraftList";
const	XMPPropertyName kXMPPropDraftProperties 	= "OpenDoc:Property:DraftProperties";
const	XMPPropertyName kXMPPropRootSU 				= "OpenDoc:Property:Draft:RootStorageUnit";

const 	XMPPropertyName	kXMPPropStorageUnitType		= "OpenDoc:Property:StorageUnit:Type";
const	XMPPropertyName kXMPPropStorageUnitName 	= "OpenDoc:Property:StorageUnitName";
const	XMPPropertyName	kXMPPropCurrentSURef 		= "OpenDoc:Property:StorageUnit:CurrentSURef";
const	XMPPropertyName kXMPPropDocumentName 		= "OpenDoc:Property:DocumentName";
const	XMPPropertyName kXMPPropDraftName 			= "OpenDoc:Property:DraftName";

const	XMPPropertyName	kXMPPropValueProperty 		= "OpenDoc:Property:Property";

// Persistent Object
const	XMPPropertyName kXMPPropName				= "OpenDoc:Property:Name";
const	XMPPropertyName kXMPPropComments			= "OpenDoc:Property:Comments";
const	XMPPropertyName	kXMPPropCreateDate 			= "OpenDoc:Property:CreationDate";
const	XMPPropertyName	kXMPPropModDate 			= "OpenDoc:Property:ModifiedDate";
const	XMPPropertyName	kXMPPropModUser 			= "OpenDoc:Property:ModifiedUser";

// WindowState
const	XMPPropertyName kXMPPropWindowList 			= "OpenDoc:Property:WindowList";

// Window
const	XMPPropertyName kXMPPropWindow 				= "OpenDoc:Property:Window";
const	XMPPropertyName kXMPPropWindowRect			= "OpenDoc:Property:WindowRect";
const	XMPPropertyName kXMPPropWindowTitle			= "OpenDoc:Property:WindowTitle";
const	XMPPropertyName kXMPPropWindowProcID		= "OpenDoc:Property:WindowProcID";
const	XMPPropertyName kXMPPropWindowIsVisible		= "OpenDoc:Property:WindowVisible";
const	XMPPropertyName kXMPPropWindowHasCloseBox	= "OpenDoc:Property:WindowHasCloseBox";
const	XMPPropertyName kXMPPropWindowHasZoomBox	= "OpenDoc:Property:WindowHasZoomBox";
const	XMPPropertyName kXMPPropWindowIsResizable	= "OpenDoc:Property:WindowIsResizable";
const	XMPPropertyName kXMPPropWindowIsRootWindow	= "OpenDoc:Property:WindowIsRootWindow";
const	XMPPropertyName kXMPPropWindowIsFloating	= "OpenDoc:Property:WindowIsFloating";
const	XMPPropertyName kXMPPropWindowHasMailer		= "OpenDoc:Property:WindowHasMailer";
const	XMPPropertyName kXMPPropWindowIsMailerAware	= "OpenDoc:Property:WindowIsMailerAware";
const	XMPPropertyName kXMPPropWindowRefCon		= "OpenDoc:Property:WindowRefCon";
const	XMPPropertyName kXMPPropRootFrame			= "OpenDoc:Property:RootFrame";
const	XMPPropertyName kXMPPropSourceFrame			= "OpenDoc:Property:SourceFrame";
const	XMPPropertyName kXMPPropShouldShowLinks		= "OpenDoc:Property:ShouldShowLinks";

// Frame
const	XMPPropertyName kXMPPropContainingFrame 	= "OpenDoc:Property:ContainingFrame";
const	XMPPropertyName kXMPPropGraphicsSystem 		= "OpenDoc:Property:GraphicsSystem";
const	XMPPropertyName kXMPPropFrameShape	 		= "OpenDoc:Property:FrameShape";
const	XMPPropertyName kXMPPropFrameRegion 		= "OpenDoc:Property:FrameRegion";
const	XMPPropertyName kXMPPropUsedRegion			= "OpenDoc:Property:UsedRegion";
const	XMPPropertyName	kXMPPropInternalTransform	= "OpenDoc:Property:InternalTransform";
const	XMPPropertyName kXMPPropPart				= "OpenDoc:Property:Part";
const	XMPPropertyName kXMPPropPartInfo			= "OpenDoc:Property:PartInfo";
const	XMPPropertyName kXMPPropViewType			= "OpenDoc:Property:ViewType";
const	XMPPropertyName kXMPPropPresentation		= "OpenDoc:Property:Presentation";
const	XMPPropertyName kXMPPropFrameGroup			= "OpenDoc:Property:FrameGroup";
const	XMPPropertyName kXMPPropIsRoot				= "OpenDoc:Property:IsRoot";
const	XMPPropertyName kXMPPropIsSubframe			= "OpenDoc:Property:IsSubFrame";
const	XMPPropertyName kXMPPropIsOverlaid			= "OpenDoc:Property:IsOverlaid";
const	XMPPropertyName kXMPPropIsFrozen			= "OpenDoc:Property:IsFrozen";
const	XMPPropertyName kXMPPropUsesCachedImage		= "OpenDoc:Property:UsesCachedImage";
const	XMPPropertyName kXMPPropDoesPropagateEvents = "OpenDoc:Property:DoesPropagateEvents";

// Draft
const XMPPropertyName	kXMPPropDraftNumber 		= "OpenDoc:Property:Draft:Number";
const XMPPropertyName	kXMPPropDraftComment 		= "OpenDoc:Property:Draft:Comment";

// Link
const	XMPPropertyName kXMPPropLink				= "OpenDoc:Property:Link";
const	XMPPropertyName kXMPPropLinkSource			= "OpenDoc:Property:LinkSource";
const	XMPPropertyName	kXMPPropLinkSpec			= "OpenDoc:Property:LinkSpec";
const	XMPPropertyName kXMPPropSourcePart			= "OpenDoc:Property:SourcePart";
const	XMPPropertyName kXMPPropEditionAlias		= "OpenDoc:Property:EditionAlias";
const	XMPPropertyName kXMPPropLinkSection			= "OpenDoc:Property:LinkSection";
const	XMPPropertyName kXMPPropLinkContentSU		= "OpenDoc:Property:LinkContentSU";
const	XMPPropertyName kXMPPropAutoExport			= "OpenDoc:Property:AutoExport";
const	XMPPropertyName kXMPPropChangeID			= "OpenDoc:Property:ChangeID";
const	XMPPropertyName kXMPPropOriginalID			= "OpenDoc:Property:OriginalID";
const	XMPPropertyName kXMPPropOriginalDraft		= "OpenDoc:Property:OriginalDraft";
const	XMPPropertyName kXMPPropOriginalCloneKind	= "OpenDoc:Property:OriginalCloneKind";

// Drag&Drop + Clipboard
const XMPPropertyName	kXMPPropContents 			= "OpenDoc:Property:Contents";
const XMPPropertyName	kXMPPropExternalTransform	= "OpenDoc:Property:ExternalTransform";
const	XMPPropertyName kXMPPropStandInFrame 		= "OpenDoc:Property:StandInFrame";

// Document Properties
const	XMPPropertyName kXMPPropEditionID			= "OpenDoc:Property:Document:EditionID";
const	XMPPropertyName kXMPPropSectionID			= "OpenDoc:Property:Document:SectionID";

// Part
// annotations
const	XMPPropertyName kXMPPropIconFamily			= "OpenDoc:Property:IconFamily";
const	XMPPropertyName kXMPPropIsStationery	 	= "OpenDoc:Property:IsStationery";
const	XMPPropertyName	kXMPPropPreferredEditor 	= "OpenDoc:Property:PreferredEditor";
const	XMPPropertyName	kXMPPropPartsBin		 	= "OpenDoc:Property:PartsBin";

// content
const	XMPPropertyName kXMPPropDisplayFrames		= "OpenDoc:Property:DisplayFrames";
const	XMPPropertyName kXMPPropEmbeddedFrames 		= "OpenDoc:Property:EmbeddedFrames";
const	XMPPropertyName kXMPPropPrintingInfo 		= "OpenDoc:Property:PrintingInfo";

#endif	// _STDPROPS_
